function spout = SpectrumWithPileup(sp,eta,varargin)
% function spout = SpectrumWithPileup(sp,eta,varargin)
%    create spectrum probability distribution function (PDF) after pileup by convolution
%  inputs:
%      sp: the input spectrum: a spectrum struct with fields
%          egys: the photon energies (keV)--assumed equally spaced
%          specnum: the number spectrum
%      eta: mean_rate*deadtime
%      minprob: (optional: 1e-3) used to set max pileup order in calculations
%      max_order2test: (optional: 20) max order to test for probability > minprob
%   outputs:
%      spout = convolved spectrum struct with same fields as above
%       output has same number of counts as input
% The energy range will be increased but same energy spacing
%  REA 2011-Jul-23 11:22 - 2011-Jul-28 10:23
%  COPYRIGHT:   Aprend Technology, 2011. All rights reserved.

nreqargs = 2;
assert(nargin>=nreqargs);
minprob = 1e-3;
max_order2test = 30;

if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'minprob';         minprob=varargin{i+1};   assert(isscalar(minprob));  i=i+1;
     case 'max_order2test';         max_order2test=varargin{i+1};   assert(isscalar(minprob));  i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

spout.egys = sp.egys(:);
spout.specnum = sp.specnum(:);
   % set maxorder

maxorder = find(poisscdfLOC(0:max_order2test, eta) > (1 - minprob), 1, 'first') - 1;
assert( ~isempty(maxorder) );
   
if  maxorder == 0;
    spout.spec_in = sp;
    spout.maxorder = maxorder;
    spout.eta = eta;
    return
end


   % normalize input spectrum to make pdf
lambda = trapz(sp.egys,sp.specnum); 
spnorm = sp.specnum/lambda;

    % min and max energy for convolved spectra
minE = min(sp.egys);
maxE = max(sp.egys);

    % energy range for accumulation array
deltaE = (sp.egys(end) - sp.egys(1))/(numel(sp.egys) - 1);
spout.egys = minE:deltaE:(maxorder + 1)*maxE;
spout.egys = spout.egys(:);


    % initialize output spectrum
ntot = numel(spout.egys);
spout.specnum = zeros(ntot,1);
idxs = round(interp1(spout.egys,1:ntot,sp.egys));
spout.specnum(idxs) = poisspdf(0,eta)*spnorm;
spconv = spnorm; % holds repeated convolutions of spnorm

   % probabilities of additional pulses in count time
  
for k = 1:maxorder
   fc = conv(spnorm,spconv);
   idxs = round(interp1(spout.egys,1:ntot, linspace((k+1)*minE, (k+1)*maxE, numel(fc)))  );
   spout.specnum(idxs) = spout.specnum(idxs) + poisspdfLOC(k,eta)*fc;
   spconv = fc;
end

   % set expected output counts per non-para;yzable formula
spout.specnum = (lambda/(1 + eta))*spout.specnum/trapz(spout.egys,spout.specnum); 
   % additional information
spout.spec_in = sp;
spout.maxorder = maxorder;
 spout.eta = eta;
 
 
 %---------- LOCAL FUNCTIONS -----------
    function pdf = poisspdfLOC(xs,eta) % Poisson probability density function
        % see Octave Poisspdf function
    assert(eta>0);
    assert(all(xs>=0));
    assert(all(xs<Inf));
    assert(all(round(xs)==xs)); % only integer arguments
    pdf = exp(log(eta)*xs - eta*ones(size(xs)) - gammaln(xs + 1));
    
 function cdf = poisscdfLOC(xs,eta) % cumulative Poisson distribution
    % see Octave poisscdf function. Uses incomplete gamma function
    assert(eta>0);
    assert(all(xs>=0));
    assert(all(xs<Inf));
    cdf = 1 - gammainc (eta, floor(xs) + 1);
